package Slapd::Backend::Test;

=head1 NAME

Slapd::Backend::Test - base class for slapd-perl tests

=cut

use utf8;
use strict;
use warnings;
use base 'Test::Class';
use POSIX qw/SIGTERM/;
use FindBin;
use IO::File;
use Net::LDAP;

sub startup : Test(startup)
{
	my $self = shift;

	my $libdir = "$FindBin::Bin/../lib";
	my $tmpdir = "/tmp/slapd-perl-test-$$";
	mkdir $tmpdir;

	my $ldapi = "$tmpdir/ldapi";
	$ldapi =~ s/\//%2f/g;
	my $host = "ldapi://$ldapi/";

	my $config_file = "$tmpdir/slapd.conf";
	my $config_content;
	my $cfg = new IO::File( "$FindBin::Bin/etc/slapd.conf.dist", '<' );
	while( defined( my $line = $cfg->getline() ) )
	{
		$config_content .= $line;
	}
	$cfg->close();
	$config_content =~ s/%%LIBDIR%%/$libdir/g;
	$config_content =~ s/%%TMPDIR%%/$tmpdir/g;
	$cfg = new IO::File( $config_file, '>' );
	$cfg->print( $config_content );
	$cfg->close();

	$self->{config_content} = $config_content;
	$self->{tmpdir} = $tmpdir;
	$self->{ldap_host} = $host;
	$self->{ldap_binddn} = $1 if $config_content =~ /^rootdn\s+"?(.+?)"?$/m;
	$self->{ldap_bindpw} = $1 if $config_content =~ /^rootpw\s+"?(.+?)"?$/m;

	if( ( my $ec = system( "/usr/local/libexec/slapd -h $host -f $config_file" ) ) == 0 )
	{
		sleep 1;
	}
	else
	{
		die "Can't execute slapd: $ec\n";
	}

	return;
}

sub shutdown : Test(shutdown)
{
	my $self = shift;

	if( my $pidfile = new IO::File( "$self->{tmpdir}/slapd.pid", '<' ) )
	{
		my $pid = $pidfile->getline();
		chomp $pid;
		$pidfile->close();
		kill SIGTERM, $pid;
		sleep 1 while kill 0, $pid;
	}

	system( "rm -rf $self->{tmpdir}" );

	return;
}

sub before : Test(setup)
{
	my $self = shift;

	$self->{ldap} = new Net::LDAP( $self->{ldap_host} );
	$self->{ldap}->bind( $self->{ldap_binddn}, password => $self->{ldap_bindpw} );

	return;
}

sub after : Test(teardown)
{
	my $self = shift;

	$self->{ldap}->unbind();

	return;
}

1;

=head1 SEE ALSO

L<slapd-perl>, L<slapd>

=head1 AUTHOR

Aleksey Mashanov <aleksey.mashanov@gmail.com>

=head1 LICENSE

This library is free software, you can redistribute it and/or modify it under
the same terms as Perl itself.

=cut
